@@ -31,12 +31,24 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 gocr_Image *currentimage = NULL;
 
+static gocr_List garbage;
+
+int _gocr_initImage ( void ) {
+	list_init(&garbage);
+	return 0;
+}
+
+void _gocr_endImage ( void ) {
+	/* todo */
+}
+
 #define FUNCTION		"datamalloc"
 /* allocates memory according to the type, and do it portably */
 static int datamalloc ( gocr_Image * image ) {
 	int			i;
 	size_t			size, psize;
 
+	_gocr_printf(3, FUNCTION, "(%p)\n", image);
 	switch (image->type) {
 	  case GOCR_NONE:
 	  case GOCR_OTHER:
@@ -44,12 +56,15 @@ static int datamalloc ( gocr_Image * image ) {
 	  case GOCR_BW:
 		size = sizeof(gocr_PixelBW);
 		psize = sizeof(gocr_PixelBW *);
+		break;
 	  case GOCR_GRAY:
 		size = sizeof(gocr_PixelGray);
 		psize = sizeof(gocr_PixelGray *);
+		break;
 	  case GOCR_COLOR:
 		size = sizeof(gocr_PixelColor);
 		psize = sizeof(gocr_PixelColor *);
+		break;
 	}
 
 	image->data.pix = (gocr_Pixel **)malloc(image->height * psize);
@@ -79,21 +94,25 @@ static int datamalloc ( gocr_Image * image ) {
   \brief Fills the NEW image, sharing it with orig->data.
 
   Long description. Mask is copied, not shared.
+  If creating a new image from a image that is already shared, the new image
+  will be considered son of the parent of that image. In other words, an image
+  either has sons or a parent. This is transparent to the user, and is much
+  better from the developer's point of view.
 
   \param orig Original image.
   \param x0,y0 A vertex of the area to be copied.
   \param x1,y1 The opposite vertex.
   \param new A pointer to the new image. May be NULL.
-  \sa _gocr_imageCopy
+  \sa _gocr_imageCopy, gocr_imageFree.
   \retval 0 OK
   \retval -1 error.
 */
 int _gocr_imageSharedCopy ( gocr_Image *orig, int x0, int y0, int x1, int y1,
-				gocr_Image *new ) {
+				gocr_Image *newimage ) {
 	int			i;
 
 	_gocr_printf(3, FUNCTION, "(%p, %d, %d, %d, %d, %p)\n", orig, x0, y0,
-		     x1, y1, new);
+		     x1, y1, newimage);
 
 	if (orig == NULL) {
 		_gocr_printf(1, FUNCTION, "NULL pointer\n");
@@ -106,10 +125,10 @@ int _gocr_imageSharedCopy ( gocr_Image *orig, int x0, int y0, int x1, int y1,
 		return -1;
 	}
 
-#ifdef notnotnot /* this would require gocr_Image **new. */
-	if (new == NULL) {
-		new = (gocr_Image *) malloc(sizeof(gocr_Image));
-		if (new == NULL) {
+#ifdef notnotnot /* this would require gocr_Image **newimage. */
+	if (newimage == NULL) {
+		newimage = (gocr_Image *) malloc(sizeof(gocr_Image));
+		if (newimage == NULL) {
 			_gocr_printf(1, FUNCTION, "NULL malloc\n");
 			return -1;
 		}
@@ -119,65 +138,77 @@ int _gocr_imageSharedCopy ( gocr_Image *orig, int x0, int y0, int x1, int y1,
 	}
 #endif
 
-	new->filename = NULL;
-	new->width = x1 - x0 + 1;
-	new->height = y1 - y0 + 1;
-	new->type = orig->type;
-	switch (new->type) {
+	newimage->filename = NULL;
+	newimage->width = x1 - x0 + 1;
+	newimage->height = y1 - y0 + 1;
+	newimage->type = orig->type;
+	switch (newimage->type) {
 	  case GOCR_NONE:
 	  case GOCR_OTHER:
-		_gocr_printf(1, FUNCTION, "type %d\n", new->type);
+		_gocr_printf(1, FUNCTION, "type %d\n", newimage->type);
 		return -1;	/* shouldn't occur */
 	  case GOCR_BW:
-		new->data.pix = malloc(new->height * sizeof(gocr_PixelBW *));
+		newimage->data.pix = malloc(newimage->height * sizeof(gocr_PixelBW *));
+		break;
 	  case GOCR_GRAY:
-		new->data.pix = malloc(new->height * sizeof(gocr_PixelGray *));
+		newimage->data.pix = malloc(newimage->height * sizeof(gocr_PixelGray *));
+		break;
 	  case GOCR_COLOR:
-		new->data.pix = malloc(new->height * sizeof(gocr_PixelColor *));
+		newimage->data.pix = malloc(newimage->height * sizeof(gocr_PixelColor *));
+		break;
 	}
-	if (new->data.pix == NULL) {
+	if (newimage->data.pix == NULL) {
 		_gocr_printf(1, FUNCTION, "NULL malloc 2\n");
 		return -1;
 	}
 	if (orig->mask) {
-		new->mask = malloc(((new->height * new->width) >> 3) + 1);
-		if (new->mask == NULL) {
+		newimage->mask = malloc(((newimage->height * newimage->width) >> 3) + 1);
+		if (newimage->mask == NULL) {
 			_gocr_printf(1, FUNCTION, "NULL malloc 3\n");
 			return -1;
 		}
-		memcpy(new->mask, orig->mask, ((new->height*new->width)>>3) + 1);
+		memcpy(newimage->mask, orig->mask, ((newimage->height*newimage->width)>>3) + 1);
 	}
 	else 
-		new->mask = NULL;
+		newimage->mask = NULL;
 		
-	for (i = 0; i < new->height; i++)
-		new->data.pix[i] = orig->data.pix[i + y0] + 
-				x0 * _gocr_imagePixelSize(new);
+	for (i = 0; i < newimage->height; i++)
+		newimage->data.pix[i] = orig->data.pix[i + y0] + 
+				x0 * _gocr_imagePixelSize(newimage);
 
+	if (orig->parent) {
+		newimage->parent = orig->parent;
+		newimage->sons = 0;
+	}
+	else {
+		newimage->parent = orig;
+		newimage->sons = 0;
+	}
+	newimage->parent->sons++;
 	return 0;
 }
 
 #undef FUNCTION
 #define FUNCTION		"_gocr_imageCopy"
 /**
-  \brief fill new image, copying the data to a new memory location
+  \brief fill newimage image, copying the data to a new memory location
 
   Long description. Copies mask.
 
   \param orig Original image.
   \param x0,y0 A vertex of the area to be copied.
   \param x1,y1 The opposite vertex.
-  \param new A pointer to the new image. May be NULL.
+  \param newimage A pointer to the new image. May not be NULL.
   \sa _gocr_imageSharedCopy
   \retval 0 OK
   \retval -1 error.
 */
 int _gocr_imageCopy ( gocr_Image *orig, int x0, int y0, int x1, int y1,
-		    gocr_Image *new ) {
+		    gocr_Image *newimage ) {
 	int			i;
 
 	_gocr_printf(3, FUNCTION, "(%p, %d, %d, %d, %d, %p)\n", orig, x0, y0, 
-			x1, y1, new);
+			x1, y1, newimage);
 
 	if (orig == NULL) {
 		_gocr_printf(1, FUNCTION, "NULL pointer\n");
@@ -191,38 +222,40 @@ int _gocr_imageCopy ( gocr_Image *orig, int x0, int y0, int x1, int y1,
 		return -1;
 	}
 
-#ifdef notnotnot /* this would require gocr_Image **new. */
-	if (new == NULL) {
-		new = (gocr_Image *)malloc(sizeof(gocr_Image));
-		if (new == NULL) {
+#ifdef notnotnot /* this would require gocr_Image **newimage. */
+	if (newimage == NULL) {
+		newimage = (gocr_Image *)malloc(sizeof(gocr_Image));
+		if (newimage == NULL) {
 			_gocr_printf(1, FUNCTION, "NULL malloc\n");
 			return -1;
 		}
 	}
 #endif
-	new->filename = NULL;
-	new->width = x1 - x0 + 1;
-	new->height = y1 - y0 + 1;
-	new->type = orig->type;
-	if (datamalloc(new) == -1) {
+	newimage->filename = NULL;
+	newimage->width = x1 - x0 + 1;
+	newimage->height = y1 - y0 + 1;
+	newimage->type = orig->type;
+	newimage->sons = 0;
+	newimage->parent = NULL;
+	if (datamalloc(newimage) == -1) {
 		_gocr_printf(1, FUNCTION, "datamalloc\n");
 		return -1;
 	}
 
 	if (orig->mask) {
-		new->mask = malloc(((new->height * new->width) >> 3) + 1);
-		if (new->mask == NULL) {
+		newimage->mask = malloc(((newimage->height * newimage->width) >> 3) + 1);
+		if (newimage->mask == NULL) {
 			_gocr_printf(1, FUNCTION, "NULL malloc 3\n");
 			return -1;
 		}
-		memcpy(new->mask, orig->mask, ((new->height * new->width) >> 3) + 1);
+		memcpy(newimage->mask, orig->mask, ((newimage->height * newimage->width) >> 3) + 1);
 	}
 	else 
-		new->mask = NULL;
+		newimage->mask = NULL;
 
-	for (i = 0; i < new->height; i++)
-		memcpy(new->data.pix[i], orig->data.pix[i + y0] +
-			       x0 * _gocr_imagePixelSize(new), new->width);
+	for (i = 0; i < newimage->height; i++)
+		memcpy(newimage->data.pix[i], orig->data.pix[i + y0] +
+			       x0 * _gocr_imagePixelSize(newimage), newimage->width);
 	return 0;
 }
 
@@ -264,6 +297,7 @@ void _gocr_fixParameters ( int *x0, int *y0, int *x1, int *y1 ) {
     Note 1: the last field must be NULL.
     Note 2: "smaller" extensions must come later: ".pnm.gz" must come
        before ".pnm".
+    todo: make this thing dynamic, and let users add new conversors.
  */
 static const char *xlist[] = {
 	".pnm.gz",    "gzip -cd",
@@ -446,6 +480,8 @@ int _gocr_imageLoad ( const char *filename, void *data ) {
 #endif
 	currentimage->filename = strdup(filename);
 	currentimage->mask = NULL;
+	currentimage->sons = 0;
+	currentimage->parent = NULL;
 	
 	fclose(fp);
 
@@ -466,40 +502,105 @@ int _gocr_imageLoad ( const char *filename, void *data ) {
 /**
   \brief Closes an image.
 
-  This function closes the image in currentimage, freeing all memory.
+  This function closes the image in currentimage, freeing all memory associated
+  with it (except block structures).
+
 */
 void gocr_imageClose ( void ) {
+	gocr_Block	*block;
+	gocr_Box	*box;
+
+	for_each_data(&blocklist) {
+		block = (gocr_Block *)list_get_current(&blocklist);
+		for_each_data(&block->boxlist) {
+			box = (gocr_Box *)list_get_current(&block->boxlist);
+			gocr_imageFree(box->image); 
+			for_each_data(&box->possible) {
+				gocr_Char *c = (gocr_Char *)list_get_current(&box->possible);
+				free(c);
+			} end_for_each(&box->possible);
+			list_free(&box->possible);
+			free(box);
+		} end_for_each(&block->boxlist);
+		list_free(&block->boxlist);
+		gocr_imageFree(block->image);
+/*		free(block); */
+	} end_for_each(&blocklist);
+	list_free(&blocklist);
+
 	gocr_imageFree(currentimage);
+	currentimage = NULL;
 }
 
+#undef FUNCTION
+#define FUNCTION			"gocr_imageFree"
 /**
   \brief Closes an image.
 
-  This function closes the image, freeing all memory.
+  This function closes the image, freeing all memory. Images that have 
+  "children", i.e., the "parent" of shared images may not be instantaneously
+  freed, and the responsability of freeing it is handled to the internal
+  garbage collector.
 
-  \bug This function won't work in shared images.
   \param image A pointer to the image structure.
 */
 void gocr_imageFree ( gocr_Image *image ) {
-	_gocr_printf(3, "gocr_imageFree", "(%p)\n", image);
+	_gocr_printf(3, FUNCTION, "(%p)\n", image);
 
 	if (image == NULL)
 		return;
 
-	if (image->filename)
-		free(image->filename);
-	if (image->data.pix) {
-		int i;
+	if (image->sons == 0) { /* no children, free it */
+		if (image->parent) { /* is shared */
+			image->parent->sons--;
 
-		for (i = 0; i < image->height; i++)
-			if (image->data.pix[i])
-				free(image->data.pix[i]);
-
-		free(image->data.pix);
+			/* garbage collection */
+			if (image->parent->sons == 0) {
+				gocr_Image *par;
+				for_each_data(&garbage) {
+					par = (gocr_Image *)list_get_current(&garbage);
+					if (par == image->parent)
+						break;
+				} end_for_each(&garbage);
+				if (par == image->parent) {
+					list_del(&garbage, par);
+					gocr_imageFree(par);
+				}
+			}
+			free(image->data.pix);
+		}
+		else { /* it is not shared */
+			if (image->data.pix) {
+				int i;
+	
+				for (i = 0; i < image->height; i++)
+					if (image->data.pix[i])
+						free(image->data.pix[i]);
+				free(image->data.pix);
+			}
+		}
+		if (image->filename)
+			free(image->filename);
+		if (image->mask)
+			free(image->mask);
+		free(image);
 	}
-	if (image->mask)
-		free(image->mask);
-	free(image);
+	else { /* has sons */
+		gocr_Image *new;
+					
+		new = (gocr_Image *)malloc(sizeof(gocr_Image));
+		if (!new) {
+			_gocr_printf(2, FUNCTION, 
+				"Could not duplicate, using the old image. This may lead to future problems.\n");
+			new = image;
+		}
+		else {
+			memcpy(new, image, sizeof(gocr_Image));
+			free(image);
+		}
+		list_app(&garbage, new);
+	}
+	return;
 }
 
 
@@ -564,15 +665,15 @@ int _gocr_imageWrite ( gocr_Image *image, char *filename, char data ) {
 			for_each_data(&block->boxlist) {
 				box = (gocr_Box *)list_get_current(&block->boxlist);
 				for (i = box->x0; i <= box->x1; i++) {
-					_gocr_private2(&block->image, i, 
+					_gocr_private2(block->image, i, 
 							box->y0) = 1;
-					_gocr_private2(&block->image, i, 
+					_gocr_private2(block->image, i, 
 							box->y1) = 1;
 				}
 				for (i = box->y0; i <= box->y1; i++) {
-					_gocr_private2(&block->image, 
+					_gocr_private2(block->image, 
 							box->x0, i) = 1;
-					_gocr_private2(&block->image, 
+					_gocr_private2(block->image, 
 							box->x1, i) = 1;
 				}
 			}
