@@ -5,18 +5,19 @@
 #include "../include/gocr.h"
 #include "../include/gocr_module.h"
 #include "../include/gocr_char.h"
-#include "../include/list.h"
+#include "../include/gocr_list.h"
 #include "../src/_gocr.h"
 
 int main (int argc, char **argv) {
   gocr_Block block;
+ char *p;
   
   if ( gocr_init(argc, argv) != 0 ) {
     printf("Could not init gocr\n");
     exit(1);
   }
 
-  gocr_setAttribute(VERBOSE, (void *)3);
+  gocr_setAttribute(VERBOSE, (void *)4);
   gocr_imageLoad("small.pbm", (void *)GOCR_BW);
 
   block.x0 = block.y0 = 0;
@@ -25,6 +26,9 @@ int main (int argc, char **argv) {
   block.t = gocr_blockTypeGetByName("TEXT");
   gocr_blockAdd(&block);
 
+p = malloc(100);
+strcpy(p, "hello");
+
   currentblock = &block;
 
   gocr_charBegin();
@@ -46,6 +50,7 @@ int main (int argc, char **argv) {
   gocr_printBox(currentbox);
 
   gocr_mainImageWriteWithData("charfind.ppm");
+  gocr_imageClose();
   gocr_finalize();
   return 0;
 }
