@@ -40,9 +40,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "pgm2asc.h"
 #include "pcx.h"
 #include "ocr0.h"		/* only_numbers */
+#include "progress.h"
 
 /* the following line is used to name the package (make dist) */
-#define version_string "0.41"
+#define version_string "0.44"
 
 static void out_version(int v) {
   fprintf(stderr, " Optical Character Recognition --- gocr " version_string "\n");
@@ -64,7 +65,8 @@ static void help(void) {
 	  " -o name   - output file  (redirection of stdout)\n"
 	  " -e name   - logging file (redirection of stderr)\n"
 	  " -x name   - progress output to fifo (see manual)\n"
-	  " -p name   - database path including final slash (default is ./db/)\n"
+	  " -p name   - database path including final slash (default is ./db/)\n");
+  fprintf(stderr, /* string length less than 509 bytes for ISO C89 */
 	  " -f fmt    - output format (ISO8859_1 TeX HTML XML UTF8 ASCII)\n"
 	  " -l num    - threshold grey level 0<160<=255 (0 = autodetect)\n"
 	  " -d num    - dust_size (remove small clusters, -1 = autodetect)\n"
@@ -73,12 +75,14 @@ static void help(void) {
 	  " -c string - list of chars (debugging, see manual)\n"
 	  " -C string - char filter (ex. hexdigits: ""0-9A-Fx"", only ASCII)\n"
 	  " -m num    - operation modes (bitpattern, see manual)\n"
-	  " -a num      value of certainty (in percent, 0..100, default=95)\n"
+	  " -a num      value of certainty (in percent, 0..100, default=95)\n");
+  fprintf(stderr, /* string length less than 509 bytes for ISO C89 */
 	  " examples:\n"
 	  "\tgocr -m 4 text1.pbm                   # do layout analyzis\n"
 	  "\tgocr -m 130 -p ./database/ text1.pbm  # extend database\n"
 	  "\tdjpeg -pnm -gray text.jpg | gocr -    # use jpeg-file via pipe\n"
 	  "\n");
+  fprintf(stderr, " webpage: http://jocr.sourceforge.net/\n");
   exit(0);
 }
 
@@ -235,10 +239,34 @@ static void mark_start(job_t *job) {
   if (job->cfg.verbose) {
     fprintf(stderr, "#");
     out_version(0);
+    /* insert some helpful info for support */
+    fprintf(stderr, "# compiled: " __DATE__ );
+#if defined(__GNUC__)
+    fprintf(stderr, " GNUC-%d", __GNUC__ );
+#endif
+#ifdef __GNUC_MINOR__
+    fprintf(stderr, ".%d", __GNUC_MINOR__ );
+#endif
+#if defined(__linux)
+    fprintf(stderr, " linux");
+#else defined(__unix)
+    fprintf(stderr, " unix");
+#endif
+#if defined(__WIN32) || defined(__WIN32__)
+    fprintf(stderr, " WIN32");
+#endif
+#if defined(__WIN64) || defined(__WIN64__)
+    fprintf(stderr, " WIN64");
+#endif
+#if defined(__VERSION__)
+    fprintf(stderr, " version " __VERSION__ );
+#endif
+    fprintf(stderr, "\n");
     fprintf(stderr,
-	    "# options are: -l %d -s %d -v %d -c %s -m %d -d %d -n %d %s\n",
+	    "# options are: -l %d -s %d -v %d -c %s -m %d -d %d -n %d -a %d\n",
 	    job->cfg.cs, job->cfg.spc, job->cfg.verbose, job->cfg.lc, job->cfg.mode, 
-            job->cfg.dust_size, job->cfg.only_numbers, job->src.fname);
+            job->cfg.dust_size, job->cfg.only_numbers, job->cfg.certainty);
+    fprintf(stderr, "# file: %s\n", job->src.fname);
 #ifdef USE_UNICODE
     fprintf(stderr,"# using unicode\n");
 #endif
@@ -263,16 +291,18 @@ static void mark_end(job_t *job) {
 #endif
 }
 
-static void read_picture(job_t *job) {
+static int read_picture(job_t *job) {
+  int rc=0;
   assert(job);
 
   if (strstr(job->src.fname, ".pcx"))
     readpcx(job->src.fname, &job->src.p, job->cfg.verbose);
   else
-    readpgm(job->src.fname, &job->src.p, job->cfg.verbose);
+    rc=readpgm(job->src.fname, &job->src.p, job->cfg.verbose);
+  return rc; /* 1 for multiple images, 0 else */
 }
 
-// subject of change, we need more output for XML (ToDo)
+/* subject of change, we need more output for XML (ToDo) */
 void print_output(job_t *job) {
   int linecounter = 0;
   const char *line;
@@ -282,7 +312,7 @@ void print_output(job_t *job) {
   linecounter = 0;
   line = getTextLine(linecounter++);
   while (line) {
-    // notice: decode() is shiftet to getTextLine since 0.38
+    /* notice: decode() is shiftet to getTextLine since 0.38 */
     fputs(line, stdout);
     if (job->cfg.out_format==HTML) fputs("<br />",stdout);
     if (job->cfg.out_format!=XML)  fputc('\n', stdout);
@@ -291,7 +321,7 @@ void print_output(job_t *job) {
   free_textlines();
 }
 
-//FIXME jb: remove JOB;
+/* FIXME jb: remove JOB; */
 job_t *JOB;
 
 
@@ -299,30 +329,36 @@ job_t *JOB;
 // ------   MAIN - replace this by your own aplication! 
 // ------------------------------------------------------------- */
 int main(int argn, char *argv[]) {
+  int multipnm=1;
   job_t job;
 
   JOB = &job;
   setvbuf(stdout, (char *) NULL, _IONBF, 0);	/* not buffered */
   
-  job_init(&job);
-  
-  process_arguments(&job, argn, argv);
-  
-  mark_start(&job);
- 
-  read_picture(&job);
-  /* separation of main and rest for using as lib
-     this will be changed later => introduction of set_option()
-     for better communication to the engine  */
+  while (multipnm==1) {
 
-  /* call main loop */
-  pgm2asc(&job);
+    job_init(&job);
+  
+    process_arguments(&job, argn, argv);
 
-  print_output(&job);
+    mark_start(&job);
+
+    multipnm = read_picture(&job);
+    /* separation of main and rest for using as lib
+       this will be changed later => introduction of set_option()
+       for better communication to the engine  */
+    if (multipnm<0) break; /* read error */
   
-  mark_end(&job);
+    /* call main loop */
+    pgm2asc(&job);
+
+    mark_end(&job);
   
-  job_free(&job);
+    print_output(&job);
+
+    job_free(&job);
+  
+  }
   
   return 0;
 }
